@@ -29,9 +29,10 @@ except ImportError:
                 sys.settrace(original_trace)
         return wrapper
 
-_PY33 = sys.version_info[:2] >= (3, 3)
+_PY343 = sys.version_info[:3] >= (3, 4, 3)
 
 from zodbpickle.pickle_3 import bytes_types
+from . import _is_pypy
 
 # Tests that try a number of pickle protocols should have a
 #     for proto in protocols:
@@ -425,7 +426,7 @@ DATA5 = (b'\x80\x02cCookie\nSimpleCookie\nq\x00)\x81q\x01U\x03key'
 
 # set([3]) pickled from 2.x with protocol 2
 DATA6 = b'\x80\x02c__builtin__\nset\nq\x00]q\x01K\x03a\x85q\x02Rq\x03.'
-
+DATA6_PYPY = b'\x80\x02c__builtin__\nset\nq\x00K\x03\x85q\x01\x85q\x02Rq\x03.'
 
 def create_data():
     c = C()
@@ -775,14 +776,12 @@ class AbstractPickleTests(unittest.TestCase):
                 u = self.loads(s)
                 self.assertEqual(t, u)
 
-    @unittest.skipUnless(_PY33, "only for Python >= 3.3")
     def test_ellipsis(self):
         for proto in protocols:
             s = self.dumps(..., proto)
             u = self.loads(s)
             self.assertEqual(..., u)
 
-    @unittest.skipUnless(_PY33, "only for Python >= 3.3")
     def test_notimplemented(self):
         for proto in protocols:
             s = self.dumps(NotImplemented, proto)
@@ -1104,10 +1103,8 @@ class AbstractPickleTests(unittest.TestCase):
     @no_tracing
     def test_bad_getattr(self):
         x = BadGetattr()
-        for proto in 0, 1:
+        for proto in (0, 1, 2):
             self.assertRaises(RuntimeError, self.dumps, x, proto)
-        # protocol 2 don't raise a RuntimeError.
-        d = self.dumps(x, 2)
 
     def test_reduce_bad_iterator(self):
         # Issue4176: crash when 4th and 5th items of __reduce__()
@@ -1146,6 +1143,10 @@ class AbstractPickleTests(unittest.TestCase):
                              "Failed protocol %d: %r != %r"
                              % (proto, obj, loaded))
 
+    @unittest.skipIf(_is_pypy,
+                     'PyPy does not guarantee the identity of strings. '
+                     'See the discussion on '
+                     'http://pypy.readthedocs.org/en/latest/cpython_differences.html#object-identity-of-primitive-values-is-and-id')
     def test_attribute_name_interning(self):
         # Test that attribute names of pickled objects are interned when
         # unpickling.
@@ -1170,7 +1171,14 @@ class AbstractPickleTests(unittest.TestCase):
         loaded = self.loads(DATA5)
         self.assertEqual(type(loaded), SimpleCookie)
         self.assertEqual(list(loaded.keys()), ["key"])
-        self.assertEqual(loaded["key"].value, "Set-Cookie: key=value")
+        if _PY343:
+            # The SimpleCookie object changed the way it gets
+            # constructed in Python 3.4.3; the old behaviour was
+            # broken.
+            # See http://bugs.python.org/issue22775
+            self.assertEqual(loaded["key"].value, "value")
+        else:
+            self.assertEqual(loaded["key"].value, "Set-Cookie: key=value")
 
     def test_pickle_to_2x(self):
         # Pickle non-trivial data with protocol 2, expecting that it yields
@@ -1179,8 +1187,14 @@ class AbstractPickleTests(unittest.TestCase):
         # bytecode that 2.x will still understand.
         dumped = self.dumps(range(5), 2)
         self.assertEqual(dumped, DATA4)
+
         dumped = self.dumps(set([3]), 2)
-        self.assertEqual(dumped, DATA6)
+        if not _is_pypy:
+            # The integer in the set is pickled differently under PyPy
+            # due to the differing identity semantics (?)
+            self.assertEqual(dumped, DATA6)
+        else:
+            self.assertEqual(dumped, DATA6_PYPY)
 
     def test_large_pickles(self):
         # Test the correctness of internal buffering routines when handling
@@ -1197,7 +1211,6 @@ class AbstractPickleTests(unittest.TestCase):
         empty = self.loads(b'\x80\x03U\x00q\x00.', encoding='koi8-r')
         self.assertEqual(empty, '')
 
-    @unittest.skipUnless(_PY33, "only for Python >= 3.3")
     def test_int_pickling_efficiency(self):
         # Test compacity of int representation (see issue #12744)
         for proto in protocols:
@@ -1800,21 +1813,88 @@ class AbstractPicklerUnpicklerObjectTests(unittest.TestCase):
         unpickler = self.unpickler_class(f)
         unpickler.noload() # read past _NOLOAD_OBJECT
         self.assertEqual(f.tell(), after['_NOLOAD_OBJECT'])
-        unpickler.noload() # read past aaa
+
+        noload = unpickler.noload() # read past aaa
+        self.assertEqual(noload, None)
         self.assertEqual(f.tell(), after['aaa'])
+
         unpickler.noload() # read past bbb
         self.assertEqual(f.tell(), after['bbb'])
-        unpickler.noload() # read past ccc
+
+        noload = unpickler.noload() # read past ccc
+        self.assertEqual(noload, ccc)
         self.assertEqual(f.tell(), after['ccc'])
-        unpickler.noload() # read past ddd
+
+        noload = unpickler.noload() # read past ddd
+        self.assertEqual(noload, ddd)
         self.assertEqual(f.tell(), after['ddd'])
-        unpickler.noload() # read past eee
+
+        noload = unpickler.noload() # read past eee
+        self.assertEqual(noload, eee)
         self.assertEqual(f.tell(), after['eee'])
-        unpickler.noload() # read past fff
+
+        noload = unpickler.noload() # read past fff
+        self.assertEqual(noload, fff)
         self.assertEqual(f.tell(), after['fff'])
-        unpickler.noload() # read past ggg
+
+        noload = unpickler.noload() # read past ggg
+        self.assertEqual(noload, ggg)
         self.assertEqual(f.tell(), after['ggg'])
 
+    def test_functional_noload_dict_subclass(self):
+        """noload() doesn't break or produce any output given a dict subclass"""
+        # See http://bugs.python.org/issue1101399
+        o = MyDict()
+        o['x'] = 1
+        f = io.BytesIO()
+        pickler = self.pickler_class(f, protocol=2)
+        pickler.dump(o)
+        f.seek(0)
+        unpickler = self.unpickler_class(f)
+        noload = unpickler.noload()
+        self.assertEqual(noload, None)
+
+
+    def test_functional_noload_list_subclass(self):
+        """noload() doesn't break or produce any output given a list subclass"""
+        # See http://bugs.python.org/issue1101399
+        o = MyList()
+        o.append(1)
+        f = io.BytesIO()
+        pickler = self.pickler_class(f, protocol=2)
+        pickler.dump(o)
+        f.seek(0)
+        unpickler = self.unpickler_class(f)
+        noload = unpickler.noload()
+        self.assertEqual(noload, None)
+
+    def test_functional_noload_dict(self):
+        """noload() implements the Python 2.6 behaviour and fills in dicts"""
+        # See http://bugs.python.org/issue1101399
+        o = dict()
+        o['x'] = 1
+        f = io.BytesIO()
+        pickler = self.pickler_class(f, protocol=2)
+        pickler.dump(o)
+        f.seek(0)
+        unpickler = self.unpickler_class(f)
+        noload = unpickler.noload()
+        self.assertEqual(noload, o)
+
+
+    def test_functional_noload_list(self):
+        """noload() implements the Python 2.6 behaviour and fills in lists"""
+        # See http://bugs.python.org/issue1101399
+        o = list()
+        o.append(1)
+        f = io.BytesIO()
+        pickler = self.pickler_class(f, protocol=2)
+        pickler.dump(o)
+        f.seek(0)
+        unpickler = self.unpickler_class(f)
+        noload = unpickler.noload()
+        self.assertEqual(noload, o)
+
 
 # Tests for dispatch_table attribute
 
